/*
** Module   : tty.keyboard.js
** Abstract : keystrokes management.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140110 First version.
*/

tty.keyboard = (function()
{
   /* exports object */
   var my = {};    
   /* track escape status */
   var escape = false;                     
   /* track alt char status */
   var altChar = false;

   /* initialize module */
   my.init = function()
   {      
      /* init mapping */   
      tty.keymap.init();
   };
                    
   /* keypress */
   my.onkeypress = function(evt)
   {
      var key = evt.charCode;
      if (key !== 0)
      {
         sendKey(key, evt);
      }
   };
              
   /* keyup */
   my.onkeyup = function(evt)
   {
      if (evt.keyCode === keys.ALT && evt.location === evt.DOM_KEY_LOCATION_RIGHT)
      {
         altChar = false;
      }
   };

   /* keydown */
   my.onkeydown = function(evt)
   {
      var key = evt.keyCode;

      if(altChar || key === keys.CTRL  || key === keys.SHIFT)
      {
         return;
      }

      if (key === keys.ALT)
      {
         if (evt.location === evt.DOM_KEY_LOCATION_RIGHT)
         {
            altChar = evt.ctrlKey;                  
         }
         return;
      }
      
      if (key === keys.ESCAPE)
      {
         evt.preventDefault();
         escape = !escape;
         return;               
      }
                               
      var p4code = -1;
      var charCode = tty.keymap.mapKey(evt);
                                       
      // Handle ALT or Escape
      if (evt.altKey || escape)
      {
         p4code = tty.keymap.xlateAlt[key];
         
         if (p4code === undefined)
         {
            if (charCode !== -1 && ((!escape && evt.altKey) || (escape && !evt.altKey)))
            {
               p4code = tty.keymap.processCtrl(evt, charCode);
               if(p4code === -1)
               {
                  p4code = charCode;
               }
               p4code |= 1024;
            }
            else
            {
               p4code = -1;
            }
         }
         else
         {
            if (!escape && evt.altKey && key === keys.DELETE)
            {
               // special case for ALT+DELETE key combo
               p4code = 512;
            }     
         }    
                    
         escape = false;
      }
      // Normal
      else
      {
         /* normal map */
         var p4code = tty.keymap.xlate[key];
         
         if (p4code === undefined)
         {
            // check CTRL
            p4code = tty.keymap.processCtrl(evt, charCode);
         }
         else
         {
            if (p4code === keys.TAB && evt.shiftKey)
            {
               p4code = 509;
            }                           
         }
      }                                                           

      if (p4code !== -1)
      {
         sendKey(p4code, evt);
      }      
   };
                                                                                          
   /* send keyEvent */
   var sendKey = function(key, evt)
   {
      var data = {};
      data.c = 0;
      data.k = key;
      tty.socket.send(JSON.stringify(data));
      evt.preventDefault();
      evt.stopImmediatePropagation();
   };
            
   /* exports */
   return my;    
}());
